<?php

use Illuminate\Support\Facades\Route;
use App\Http\Controllers\PlansController;
use App\Http\Controllers\CRUD\PostsController;
use App\Http\Controllers\FrontEndController;
use App\Http\Controllers\CompaniesController;
use Spatie\WelcomeNotification\WelcomesNewUsers;
use App\Http\Controllers\Auth\MyWelcomeController;
use App\Http\Controllers\Auth\SocialController;

/*
|--------------------------------------------------------------------------
| Web Routes
|--------------------------------------------------------------------------
|
| Here is where you can register web routes for your application. These
| routes are loaded by the RouteServiceProvider within a group which
| contains the "web" middleware group. Now create something great!
|
*/

Route::get('/',  [FrontEndController::class, 'index'])->name('landing');
Route::get('/new',  [FrontEndController::class, 'register'])->name('newcompany.register');
Route::get('/'.config('settings.url_route','company').'/{alias}', [FrontEndController::class, 'company'])->name('vendor');
Route::get('/notify/{type}/{id}/{message}', [CompaniesController::class, 'notify'])->name('company.notify');
Route::group(['middleware' => ['web', WelcomesNewUsers::class,]], function () {
    Route::get('welcome/{user}', [MyWelcomeController::class, 'showWelcomeForm'])->name('welcome');
    Route::post('welcome/{user}', [MyWelcomeController::class, 'savePassword']);
});

//AUTH
Route::group(['middleware' => ['web']], function () {
    Route::get('/login/google',  [SocialController::class, 'googleRedirectToProvider'])->name('google.login');
    Route::get('/login/google/redirect', [SocialController::class, 'googleHandleProviderCallback']);
    Route::get('/login/facebook', 'App\Http\Controllers\Auth\SocialController@facebookRedirectToProvider')->name('facebook.login');
    Route::get('/login/facebook/redirect', [SocialController::class, 'facebookHandleProviderCallback']);
});

Route::middleware(['web','auth','impersonate'])->group(function () {
    Route::get('/dashboard/{lang?}', 'App\Http\Controllers\DashboardController@dashboard')->name('dashboard');
    Route::get('/home/{lang?}', 'App\Http\Controllers\DashboardController@dashboard')->name('home');

    Route::name('admin.')->group(function () {
        Route::resource(config('settings.url_route_plural','companies'), 'App\Http\Controllers\CompaniesController',[
            'names' => [
                'index' => 'companies.index',
                'store' => 'companies.store',
                'edit' => 'companies.edit',
                'create' => 'companies.create',
                'destroy' => 'companies.destroy',
                'update' => 'companies.update',
                'show' => 'companies.show'
            ]
        ]);
        
        
        //Other companies routes
        Route::get('removecompany/{company}', 'App\Http\Controllers\CompaniesController@remove')->name('company.remove');
        Route::get('/company/{company}/activate', 'App\Http\Controllers\CompaniesController@activateCompany')->name('company.activate');
        Route::put('companies_app_update/{company}', 'App\Http\Controllers\CompaniesController@updateApps')->name('company.updateApps');
        Route::get('companies/loginas/{company}', 'App\Http\Controllers\CompaniesController@loginas')->name('companies.loginas');
        Route::get('stopimpersonate', 'App\Http\Controllers\CompaniesController@stopImpersonate')->name('companies.stopImpersonate');
        Route::get('/share', 'App\Http\Controllers\CompaniesController@share')->name('share');

        Route::resource('settings', 'App\Http\Controllers\SettingsController');

         // Landing page settings 
         Route::get('landing', 'App\Http\Controllers\SettingsController@landing')->name('landing');
         Route::controller(PostsController::class)->prefix('landing')->name('landing.')->group(function () {
             Route::get('posts/{type}', 'index')->name('posts');
             Route::get('posts/{type}/create', 'create')->name('posts.create');
             Route::post('posts/{type}', 'store')->name('posts.store');
            
             Route::get('posts/edit/{post}', 'edit')->name('posts.edit');
             Route::put('posts/{post}', 'update')->name('posts.update');
             Route::get('posts/del/{post}', 'destroy')->name('posts.delete');
 
         });
        
         //Apps
         Route::get('apps','App\Http\Controllers\AppsController@index')->name('apps.index');
         Route::get('appremove/{alias}','App\Http\Controllers\AppsController@remove')->name('apps.remove');
         Route::post('apps','App\Http\Controllers\AppsController@store')->name('apps.store');

    });

    Route::resource('plans',PlansController::class);
    Route::controller(PlansController::class)->group(function () {  
        Route::get('/plan', 'current')->name('plans.current');
        Route::post('/subscribe/plan', 'subscribe')->name('plans.subscribe');
        Route::get('/subscribe/cancel', 'cancelStripeSubscription')->name('plans.cancel');
        Route::get('/subscribe/plan3d/{plan}/{user}', 'subscribe3dStripe')->name('plans.subscribe_3d_stripe');
        Route::post('/subscribe/update', 'adminupdate')->name('update.plan');
    });
    
});
